#ifndef __XP_PARAM_H__
#define __XP_PARAM_H__

/*	Copyright (C) 2001,2002,2003 Ronnie Maor and Michael Brand
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * As a special exception, the copyright holders give permission
 * for additional uses of the text contained in its release of XPARAM.
 *
 * The exception is that, if you link the XPARAM library with other files
 * to produce an executable, this does not by itself cause the
 * resulting executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of
 * linking the XPARAM library code into it.
 *
 * This exception does not however invalidate any other reasons why
 * the executable file might be covered by the GNU General Public License.
 *
 * If you write modifications of your own for XPARAM, it is your choice
 * whether to permit this exception to apply to your modifications.
 * If you do not wish that, delete this exception notice.  
 */

#include <string>
#include "xp_config.h"
#include "xp_handle.h"
#include "xp_value.h"

namespace xParam_internal {

	class ParamSet;

	class Param {
	public:
		virtual ~Param() {}

		virtual bool is_input() const = 0;
		virtual bool is_output() const = 0;

		virtual bool was_assigned_to() const = 0;
	protected:
		virtual Handle<Value> get_value() const = 0;
		virtual void set_value(const Handle<Value>& val) = 0;

	public:
		virtual bool has_default() const = 0;
	protected:
		virtual Handle<Value> get_default() const = 0;
		virtual void set_default(const Handle<Value>& val) = 0;

	public:
		virtual std::string name() const = 0;
                virtual std::string description() const =0;
		virtual const std::type_info& static_type_info() const = 0;
		virtual bool has_given_value() const = 0;
	private:
		friend class ParamSet;
	};	
    
} // namespace xParam_internal
  
#endif // __XP_PARAM_H__

